from __future__ import print_function
from __future__ import unicode_literals
from builtins import object
import time
from pcs_proxy.common import settings

##
## Common code shared by Collabora PCS-to-* services
## (other classes derive from this, it isn't used directly)
##


class CollaboraServiceProxyLogic(object):
    user_agent_timing = 0
    service_name = "Base Service (should never see this)"
    login_session = ""
    service_hostname = ""

    # Constructor
    #
    # In this case, all we do is mark when the user should be forced to re-enter
    # their credentials due to inactivity
    def __init__(self):
        self.user_agent_timing = int(time.time()) + 3600

    # This is a function that subclassing classes can leverage to validate or
    # reject the validity of a requested URL
    def prevalidate_request(self, validate_URI):
        return "OK"

    # This is a function that subclassing classes can leverage to refresh API
    # tokens if they are potentially expired
    def session_ensure(self):
        pass

    # Check if the user has left the PCS proxy idle for a while, if they have we
    # want to verify that they really want to start working with the proxy again
    # and if they do we also ask them to revalidate their credentials
    def require_authentication(self):
        # require_authentication should only exit once we have
        # valid-looking credentials
        if (int(time.time())) > self.user_agent_timing:
            confirms = "n"
            while not confirms.lower().strip() == "y":
                print("You haven't sent data to " + self.service_name + " in a while")
                if settings['Authenticator'].keyring_data:
                    confirms = settings['Authenticator'].get_nonblank_user_input("Are you sure you want to continue (y/n)? ")
                    if not confirms.lower().strip() == "y":
                        print("You must confirm this to continue!")
                else:
                    failure = False
                    cred_ok = False
                    while not cred_ok:
                        if failure:
                            print("Sorry, try again.\n")
                        print("Please confirm your credentials:")
                        failure = (settings['Authenticator'].reconfirm_credentials() is not True)
                        cred_ok = not failure
                    break
        else:
            pass
        self.user_agent_timing = int(time.time()) + 3600

    # If the service can logically test the credentials, this
    # function should be implemented to test those credentials
    # provided by the user and return True or False depending
    # on the tested validity of the credentials
    def test_authentication(self):
        return True

    # This is a "nicety" function that lets the user know what service the PCS
    # proxy is currently engaged to talk with
    def print_service_banner(self):
        print("Service module active: " + self.service_name)

    # A subclass-oriented function that handles as-needed mangling of incoming
    # client requests
    def filter_request(self, args):
        pass
